@@ -1,6 +1,6 @@
 # Natural Language Toolkit: Interface to Megam Classifier
 #
-# Copyright (C) 2001-2014 NLTK Project
+# Copyright (C) 2001-2015 NLTK Project
 # Author: Edward Loper <edloper@gmail.com>
 # URL: <http://nltk.org/>
 # For license information, see LICENSE.TXT
@@ -24,8 +24,6 @@ for details.
 """
 from __future__ import print_function
 
-import os
-import os.path
 import subprocess
 
 from nltk import compat
@@ -99,8 +97,9 @@ def write_megam_file(train_toks, encoding, stream,
     # Write the file, which contains one line per instance.
     for featureset, label in train_toks:
         # First, the instance number (or, in the weighted multiclass case, the cost of each label).
-        if hasattr(encoding,'cost'):
-            stream.write(':'.join(str(encoding.cost(featureset, label, l)) for l in labels))
+        if hasattr(encoding, 'cost'):
+            stream.write(':'.join(str(encoding.cost(featureset, label, l))
+                                  for l in labels))
         else:
             stream.write('%d' % labelnum[label])
 
