@@ -1,30 +1,31 @@
 # Natural Language Toolkit: Corpus Reader Utilities
 #
-# Copyright (C) 2001-2012 NLTK Project
-# Author: Steven Bird <sb@ldc.upenn.edu>
-#         Edward Loper <edloper@gradient.cis.upenn.edu>
-# URL: <http://www.nltk.org/>
+# Copyright (C) 2001-2015 NLTK Project
+# Author: Steven Bird <stevenbird1@gmail.com>
+#         Edward Loper <edloper@gmail.com>
+# URL: <http://nltk.org/>
 # For license information, see LICENSE.TXT
 
 import os
-import sys
 import bisect
 import re
 import tempfile
-try: import cPickle as pickle
-except ImportError: import pickle
-from itertools import islice
+from functools import reduce
+try:
+    import cPickle as pickle
+except ImportError:
+    import pickle
 
 # Use the c version of ElementTree, which is faster, if possible:
 try: from xml.etree import cElementTree as ElementTree
 except ImportError: from xml.etree import ElementTree
 
+from nltk.compat import string_types, text_type
 from nltk.tokenize import wordpunct_tokenize
 from nltk.internals import slice_bounds
 from nltk.data import PathPointer, FileSystemPathPointer, ZipFilePathPointer
 from nltk.data import SeekableUnicodeStreamReader
-from nltk.sourcedstring import SourcedStringStream
-from nltk.util import AbstractLazySequence, LazySubsequence, LazyConcatenation
+from nltk.util import AbstractLazySequence, LazySubsequence, LazyConcatenation, py25
 
 ######################################################################
 #{ Corpus View
@@ -123,7 +124,7 @@ class StreamBackedCorpusView(AbstractLazySequence):
        block; and tokens is a list of the tokens in the block.
     """
     def __init__(self, fileid, block_reader=None, startpos=0,
-                 encoding=None, source=None):
+                 encoding='utf8'):
         """
         Create a new corpus view, based on the file ``fileid``, and
         read with ``block_reader``.  See the class documentation
@@ -141,11 +142,6 @@ class StreamBackedCorpusView(AbstractLazySequence):
             read the file's contents.  If no encoding is specified,
             then the file's contents will be read as a non-unicode
             string (i.e., a str).
-
-        :param source: If specified, then use an ``SourcedStringStream``
-            to annotate all strings read from the file with
-            information about their start offset, end ofset,
-            and docid.  The value of ``source`` will be used as the docid.
         """
         if block_reader:
             self.read_block = block_reader
@@ -153,7 +149,6 @@ class StreamBackedCorpusView(AbstractLazySequence):
         self._toknum = [0]
         self._filepos = [startpos]
         self._encoding = encoding
-        self._source = source
         # We don't know our length (number of tokens) yet.
         self._len = None
 
@@ -180,7 +175,7 @@ class StreamBackedCorpusView(AbstractLazySequence):
                 self._eofpos = self._fileid.file_size()
             else:
                 self._eofpos = os.stat(self._fileid).st_size
-        except Exception, exc:
+        except Exception as exc:
             raise ValueError('Unable to open or access %r -- %s' %
                              (fileid, exc))
 
@@ -217,8 +212,6 @@ class StreamBackedCorpusView(AbstractLazySequence):
                 open(self._fileid, 'rb'), self._encoding)
         else:
             self._stream = open(self._fileid, 'rb')
-        if self._source is not None:
-            self._stream = SourcedStringStream(self._stream, self._source)
 
     def close(self):
         """
@@ -259,7 +252,7 @@ class StreamBackedCorpusView(AbstractLazySequence):
                 return self._cache[2][i-offset]
             # Use iterate_from to extract it.
             try:
-                return self.iterate_from(i).next()
+                return next(self.iterate_from(i))
             except StopIteration:
                 raise IndexError('index out of range')
 
@@ -420,11 +413,11 @@ def concat(docs):
     if len(docs) == 0:
         raise ValueError('concat() expects at least one object!')
 
-    types = set([d.__class__ for d in docs])
+    types = set(d.__class__ for d in docs)
 
     # If they're all strings, use string concatenation.
-    if types.issubset([str, unicode, basestring]):
-        return reduce((lambda a,b:a+b), docs, '')
+    if all(isinstance(doc, string_types) for doc in docs):
+        return ''.join(docs)
 
     # If they're all corpus views, then use ConcatenatedCorpusView.
     for typ in types:
@@ -473,14 +466,11 @@ class PickleCorpusView(StreamBackedCorpusView):
     don't want to repeat it); but the corpus is too large to store in
     memory.  The following example illustrates this technique:
 
-    .. doctest::
-        :options: +SKIP
-
         >>> from nltk.corpus.reader.util import PickleCorpusView
         >>> from nltk.util import LazyMap
-        >>> feature_corpus = LazyMap(detect_features, corpus)
-        >>> PickleCorpusView.write(feature_corpus, some_fileid)
-        >>> pcv = PickleCorpusView(some_fileid)
+        >>> feature_corpus = LazyMap(detect_features, corpus) # doctest: +SKIP
+        >>> PickleCorpusView.write(feature_corpus, some_fileid)  # doctest: +SKIP
+        >>> pcv = PickleCorpusView(some_fileid) # doctest: +SKIP
     """
     BLOCK_SIZE = 100
     PROTOCOL = -1
@@ -518,7 +508,7 @@ class PickleCorpusView(StreamBackedCorpusView):
 
     @classmethod
     def write(cls, sequence, output_file):
-        if isinstance(output_file, basestring):
+        if isinstance(output_file, string_types):
             output_file = open(output_file, 'wb')
         for item in sequence:
             pickle.dump(item, output_file, cls.PROTOCOL)
@@ -539,7 +529,7 @@ class PickleCorpusView(StreamBackedCorpusView):
             cls.write(sequence, output_file)
             output_file.close()
             return PickleCorpusView(output_file_name, delete_on_gc)
-        except (OSError, IOError), e:
+        except (OSError, IOError) as e:
             raise ValueError('Error while creating temp file: %s' % e)
 
 
@@ -653,7 +643,7 @@ def read_sexpr_block(stream, block_size=16384, comment_char=None):
     start = stream.tell()
     block = stream.read(block_size)
     encoding = getattr(stream, 'encoding', None)
-    assert encoding is not None or isinstance(block, str)
+    assert encoding is not None or isinstance(block, text_type)
     if encoding not in (None, 'utf-8'):
         import warnings
         warnings.warn('Parsing may fail, depending on the properties '
@@ -685,7 +675,7 @@ def read_sexpr_block(stream, block_size=16384, comment_char=None):
 
             # Return the list of tokens we processed
             return tokens
-        except ValueError, e:
+        except ValueError as e:
             if e.args[0] == 'Block too small':
                 next_block = stream.read(block_size)
                 if next_block:
@@ -756,11 +746,15 @@ def find_corpus_fileids(root, regexp):
         items = [name for name in fileids if re.match(regexp, name)]
         return sorted(items)
 
-    # Find fileids in a directory: use os.walk to search all
-    # subdirectories, and match paths against the regexp.
+    # Find fileids in a directory: use os.walk to search all (proper
+    # or symlinked) subdirectories, and match paths against the regexp.
     elif isinstance(root, FileSystemPathPointer):
         items = []
-        for dirname, subdirs, fileids in os.walk(root.path):
+        # workaround for py25 which doesn't support followlinks
+        kwargs = {}
+        if not py25():
+            kwargs = {'followlinks': True}
+        for dirname, subdirs, fileids in os.walk(root.path, **kwargs):
             prefix = ''.join('%s/' % p for p in _path_from(root.path, dirname))
             items += [prefix+fileid for fileid in fileids
                       if re.match(regexp, prefix+fileid)]
