@@ -770,6 +770,3 @@ def demo():
     Note that result is very poor because of only one training example.
     """
 
-if __name__ == '__main__':
-    import doctest
-    doctest.testmod(optionflags=doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS)
