@@ -458,6 +458,3 @@ def demo(text=None):
     pylab.show()
 
 
-if __name__ == "__main__":
-    import doctest
-    doctest.testmod(optionflags=doctest.NORMALIZE_WHITESPACE)
